#include <iostream>
#include <memory>
#include <vector>

class Animal {
public:
	virtual void makeSound() = 0;

	virtual ~Animal() = default;
};

class Dog : public Animal {
public:
	void makeSound() override { std::cout << "Whoof!\n"; }
};

class Cat : public Animal {
public:
	void makeSound() override { std::cout << "Meow!\n"; }
};

class SmallFurryAnimalFromAlphaCentauri : public Animal {
public:
	void makeSound() override { std::cout << "Qwxrq!\n"; }
};

int main() {
	// Animal is an abstract class, we cannot instantiate it:
	// Animal a;
	std::vector<std::unique_ptr<Animal>> animals;
	animals.push_back(std::make_unique<Dog>());
	animals.push_back(std::make_unique<Cat>());
	animals.push_back(std::make_unique<SmallFurryAnimalFromAlphaCentauri>());

	for (const auto& animal : animals) {
		animal->makeSound();
	}
}
